@@ -1,9 +1,9 @@
 import importlib
-from .environment import Environment
+#from .environment import Environment
 
 
 environments = {}
-for mod_name in ['simple_arm', 'pendulum']:
+for mod_name in ['simple_arm', 'pendulum', 'npendulum']:
     module = importlib.import_module('explauto.environment.{}'.format(mod_name))
     env = getattr(module, 'environment')
     conf = getattr(module, 'configurations')
