@@ -1,14 +1,17 @@
 import numpy as np
 
-from .. import Environment
+from ..environment import Environment
 from ...utils import bounds_min_max
 
 
 def forward(angles, lengths):
     """ Link object as defined by the standard DH representation.
+
     :param list angles: angles of each joint
+
     :param list lengths: length of each segment
-    :return tuple: x, y end-effector postion
+
+    :returns: a tuple (x, y) of the end-effector position
 
     .. warning:: angles and lengths should be the same size.
     """
@@ -18,9 +21,12 @@ def forward(angles, lengths):
 
 def joint_positions(angles, lengths):
     """ Link object as defined by the standard DH representation.
+
     :param list angles: angles of each joint
+
     :param list lengths: length of each segment
-    :return tuple: x positions of each joint, y positions of each joints, except the first one wich is fixed at (0, 0)
+
+    :returns: x positions of each joint, y positions of each joints, except the first one wich is fixed at (0, 0)
 
     .. warning:: angles and lengths should be the same size.
     """
@@ -53,25 +59,21 @@ class SimpleArmEnvironment(Environment):
         # self.readable = range(self.conf.m_ndims + self.conf.s_ndims)
         # self.writable = range(self.conf.m_ndims)
 
-    def compute_motor_command(self, ag_state):
-        motor_cmd = ag_state
-        return bounds_min_max(motor_cmd, self.conf.m_mins, self.conf.m_maxs)
+    def compute_motor_command(self, joint_pos_ag):
+        return bounds_min_max(joint_pos_ag, self.conf.m_mins, self.conf.m_maxs)
 
-    def compute_sensori_effect(self):
-        res = np.array(forward(self.state[:self.conf.m_ndims], self.lengths))
-        res += self.noise * np.random.randn(*res.shape)
-        return res
+    def compute_sensori_effect(self, joint_pos_env):
+        hand_pos = np.array(forward(joint_pos_env, self.lengths))
+        hand_pos += self.noise * np.random.randn(*hand_pos.shape)
+        return hand_pos
 
-    def plot_arm(self, ax, m, **kwargs_plot):
+    def plot_arm(self, ax, m_, **kwargs_plot):
+        m = self.compute_motor_command(m_)
         x, y = joint_positions(m, self.lengths)
         x, y = [np.hstack((0., a)) for a in x, y]
-
-        ax.plot(x, y, **kwargs_plot)
-        ax.plot(x[0], y[0], 'o', ms=6)
-        ax.plot(x[-1], y[-1], 's', ms=6)
+        ax.plot(x, y, 'grey', lw=2, **kwargs_plot)
+        ax.plot(x[0], y[0], 'ok', ms=6)
+        ax.plot(x[-1], y[-1], 'sk', ms=6)
         ax.axis([self.conf.s_mins[0], self.conf.s_maxs[0], self.conf.s_mins[1], self.conf.s_maxs[1]])
         ax.set_xlabel('X')
         ax.set_ylabel('Y')
-
-
-
